﻿/*
 *  Caching is a meant as a non-web alternative to the somewhat similar HttpCache.
 *  Copyright (C) 2009 Steffen Skov

 *  This file is part of Caching.

 *  Caching is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.

 *  Caching is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.

 *  You should have received a copy of the GNU Lesser General Public License
 *  along with Caching.  If not, see <http://www.gnu.org/licenses/>.
 */
using System;
using System.Collections.Generic;
using System.Threading;

namespace Ckode.Caching
{
    public abstract class BaseCache
    {
        private const int cleanupInterval = 60000;
        private static IList<WeakReference> cachesToCleanup;
        static BaseCache()
        {
            cachesToCleanup = new List<WeakReference>();
            Thread cleanupThread = new Thread(CleanupCachesThread);
            cleanupThread.IsBackground = true;
            cleanupThread.Priority = ThreadPriority.BelowNormal;
            cleanupThread.Start();
        }

        private static void CleanupCachesThread()
        {
            while (true)
            {
                Thread.Sleep(cleanupInterval);
                RemoveGarbageCollectedCaches();
                CleanupCaches();
            }
        }

        private static void RemoveGarbageCollectedCaches()
        {
            IList<WeakReference> referencesToRemove = GetReferencesToRemove();
            foreach (WeakReference reference in referencesToRemove)
            {
                cachesToCleanup.Remove(reference);
            }
        }

        private static IList<WeakReference> GetReferencesToRemove()
        {
            IList<WeakReference> referencesToRemove = new List<WeakReference>();
            for (int i = 0; i < cachesToCleanup.Count; i++)
            {
                WeakReference reference = cachesToCleanup[i];
                BaseCache cache = GetCache(reference);
                if (cache == null)
                    referencesToRemove.Add(reference);
            }
            return referencesToRemove;
        }

        private static void CleanupCaches()
        {
            for (int i = 0; i < cachesToCleanup.Count; i++)
            {
                WeakReference reference = cachesToCleanup[i];
                BaseCache cache = GetCache(reference);
                if (cache != null)
                    cache.RemoveExpiredItems();
            }
        }

        private static BaseCache GetCache(WeakReference reference)
        {
            return reference.Target as BaseCache;
        }

        public BaseCache()
        {
            cachesToCleanup.Add(new WeakReference(this));
        }

        protected abstract void RemoveExpiredItems();
    }
}
