﻿/*
 *  Caching is a meant as a non-web alternative to the somewhat similar HttpCache.
 *  Copyright (C) 2009 Steffen Skov

 *  This file is part of Caching.

 *  Caching is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.

 *  Caching is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.

 *  You should have received a copy of the GNU Lesser General Public License
 *  along with Caching.  If not, see <http://www.gnu.org/licenses/>.
 */
using System;
using System.Collections.Generic;

namespace Ckode.Caching
{
    public class Cache<TKey, TValue> : BaseCache where TValue : class
    {
        private Dictionary<TKey, CacheItem<TValue>> content;
        private object contentLock;

        public Cache()
        {
            content = new Dictionary<TKey, CacheItem<TValue>>();
            contentLock = new object();
        }

        public void Set(TKey key, TValue item, ExpirationType expirationType, TimeSpan expirationTime)
        {
            CacheItem<TValue> cacheItem = new CacheItem<TValue>(item, expirationType, expirationTime);
            lock (contentLock)
            {
                content[key] = cacheItem;
            }
        }

        public TValue this[TKey key]
        {
            get
            {
                return Get(key);
            }
        }

        public TValue Get(TKey key)
        {
            CacheItem<TValue> cacheItem = GetCacheItem(key);
            if (cacheItem == null || cacheItem.IsExpired())
            {
                Remove(key);
                return null;
            }
            else
                return GetValue(cacheItem);
        }

        protected override void RemoveExpiredItems()
        {
            IList<TKey> itemsToRemove = GetItemsToRemove();
            foreach (TKey key in itemsToRemove)
            {
                Remove(key);
            }
        }

        private IList<TKey> GetItemsToRemove()
        {
            IList<TKey> itemsToRemove = new List<TKey>();
            lock (contentLock)
            {
                foreach (KeyValuePair<TKey, CacheItem<TValue>> item in content)
                {
                    CacheItem<TValue> cacheItem = GetCacheItem(item.Key);
                    if (cacheItem == null || cacheItem.IsExpired())
                        itemsToRemove.Add(item.Key);
                }
            }
            return itemsToRemove;
        }

        private CacheItem<TValue> GetCacheItem(TKey key)
        {
            CacheItem<TValue> item;
            if (content.TryGetValue(key, out item))
                return item;
            return null;
        }

        public void Remove(TKey key)
        {
            lock (contentLock)
            {
                content.Remove(key);
            }
        }

        private TValue GetValue(CacheItem<TValue> cacheItem)
        {
            cacheItem.ExtendExpirationIfSliding();
            return cacheItem.Value;
        }

        public void Flush()
        {
            lock (contentLock)
            {
                content.Clear();
            }
        }
    }
}